package uk.co.badgersinfoil.metaas;

/**
 * Obtained from {@link StatementContainer#newIf(String)}, an ASIfStatement
 * allows statements to be added to the 'then' and else' branches.
 * 
 * <p>e.g. To simply add statements to the 'then' branch (executed when
 * the condition holds true),</p>
 * <pre>
 * ASIfStatement ifStmt = method.newIf("test()");
 * isStmt.addStmt("trace('test succeeded')");
 * </pre>
 * <p>will result in ActionScript code like,</p>
 * <pre>
 * if (test()) {
 * 	trace('test succeeded');
 * }
 * </pre>
 * 
 * <p>To add code to both 'then' and 'else' branches,</p>
 * <pre>
 * ASIfStatement ifStmt = method.newIf("test()");
 * isStmt.addStmt("trace('test succeeded')");
 * ifStmt.getElse().addStmt("trace('test failed')");
 * </pre>
 * <p>will result in ActionScript code like,</p>
 * <pre>
 * if (test()) {
 * 	trace('test succeeded');
 * } else {
 * 	trace('test failed');
 * }
 * </pre>
 * 
 * <p>Note that the first call to getElse() will cause the else-block to be
 * created (even if no statements are subsequently added to it).  Subsequent
 * calls to getElse() will return references to the same else-block.</p>
 */
public interface ASIfStatement extends StatementContainer {

	/**
	 * Returns a reference to an object which can populate the else-clause
	 * of this ActionScript if-statement with new code.
	 */
	public ASBlock getElse();

	public void setThen(ASBlock thenBlock);
}