package uk.co.badgersinfoil.metaas;


/**
 * A member of a type; an {@link ASMethod} or {@link ASField}.
 */
public interface ASMember {
	/**
	 * Returns the name of this field.
	 */
	public String getName();

	/**
	 * Changes the name of this ActionScript field to the given value.
	 */
	public void setName(String string);

	/**
	 * Returns the name of the return type of value this ActionScript
	 * field may contain, or null if it is untyped.
	 */
	public String getType();

	/**
	 * Defines the name of the type of object this ActionScript field may
	 * contain.  May be set to null, denoting that the value is
	 * untyped.
	 */
	public void setType(String string);

	/**
	 * Returns a value representing any protection-against-access defined
	 * for this ActionScript field.
	 */
	public Visibility getVisibility();

	/**
	 * Defines the level of protection-against-external-access for this
	 * ActionScript field.
	 */
	public void setVisibility(Visibility visibility);


	/**
	 * Returns true if this ActionScript field is static (i.e. the field
	 * definition uses the <code>static</code> keyword).
	 */
	public boolean isStatic();

	/**
	 * Defines this method to be static, or not.  If set to true, this
	 * member definition will be qualified with the <code>static</code>
	 * keyword, if false, the keyword will be absent.
	 */
	public void setStatic(boolean s);

	/**
	 * Returns the contents of the documentation comment for this method.
	 * Lines of text in the comment will alreeady have had leading '*'
	 * characters removed.  If there is no current documentation comment,
	 * null will be returned.
	 */
	public String getDocComment();

	/**
	 * Set the contents of the 'documentation comment' for this ActionScript
	 * method.  If the given text has multiple lines, each line will have
	 * initial indentation and a '*' character added, so you shouldn't
	 * include these in the text yourself.  Supplying a null value removes
	 * any existing comment.
	 */
	public void setDocComment(String text);
}