package uk.co.badgersinfoil.metaas;

import java.util.List;


/**
 * A package-declaration block within an ActionScript compilation-unit.
 */
public interface ASPackage {

	/**
	 * Return the name of this package-block, or null if no name is present.
	 * For example, for this package block:
	 * <pre>package foo.bar {
	 * 	// ...
	 * }</pre>
	 * getName() will return the value "foo.bar".
	 */
	public String getName();

	/**
	 * Sets the name of this package.  To remove the package name, set this
	 * value to null (<em>not</em> an empty string).
	 */
	public void setName(String name);
	
	public ASType getType();

	/**
	 * Returns a list of strings specifying the names which are imported
	 * into this package by package-level import statements.
	 */
	public List findImports();

	/**
	 * Adds an import statement to this package block.
	 */
	public void addImport(String name);

	/**
	 * Removes an import statement from this package block.
	 */
	public boolean removeImport(String name);
}