package uk.co.badgersinfoil.metaas;

import uk.co.badgersinfoil.metaas.impl.ASTASParser;
import uk.co.badgersinfoil.metaas.impl.ASTASProject;
import uk.co.badgersinfoil.metaas.impl.ASTASWriter;
import uk.co.badgersinfoil.metaas.impl.ASTFactory;
import uk.co.badgersinfoil.metaas.impl.ASTStatementList;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;

/**
 * Core class providing access to metaas functionality.
 */
public class ASSourceFactory {

	/**
	 * Creates a new CompilationUnit which defines a class with the given
	 * name.  To populate the new class, you can do something like,
	 * <pre>
	 * CompilationUnit cu = fact.newClass("MyTest");
	 * ASTClassType myclass = (ASTClassType)cu.getType();
	 * // ... add stuff to myclass  ...
	 * </pre>
	 */
	public CompilationUnit newClass(String qualifiedClassName) {
		return ASTFactory.synthesizeClass(qualifiedClassName);
	}

	/**
	 * Creates a new CompilationUnit which defines an interface with the
	 * given name.  To populate the new interface, you can do something
	 * like,
	 * <pre>
	 * CompilationUnit cu = fact.newClass("MyTest");
	 * ASTClassType myiface = (ASTInterfaceType)cu.getType();
	 * // ... add stuff to myiface  ...
	 * </pre>
	 */
	public CompilationUnit newInterface(String qualifiedInterfaceName) {
		return ASTFactory.synthesizeInterface(qualifiedInterfaceName);
	}

	public ASWriter newWriter() {
		return new ASTASWriter();
	}
	
	public ASParser newParser() {
		return new ASTASParser();
	}

	/**
	 * Escape the given String and place within double quotes so that it
	 * will be a valid ActionScript string literal.
	 */
	public static String str(String str) {
		StringBuffer result = new StringBuffer("\"");
		for (int i=0; i<str.length(); i++) {
			char c = str.charAt(i);
			switch (c) {
			    case '\n':
				result.append("\\n");
				break;
			    case '\t':
				result.append("\\t");
				break;
			    case '\r':
				result.append("\\r");
				break;
			    case '"':
				result.append("\\\"");
				break;
			    case '\\':
				result.append("\\\\");
				break;
			    default:
				result.append(c);
			}
		}
		result.append('"');
		return result.toString();
	}

	public ASBlock newBlock() {
		LinkedListTree block = ASTFactory.newBlock();
		return new ASTStatementList(block);
	}

	public ASProject newEmptyASProject(String outputLocation) {
		ASTASProject project = new ASTASProject(this);
		project.setOutputLocation(outputLocation);
		return project;
	}
}