package uk.co.badgersinfoil.metaas;

/**
 * Obtained from {@link StatementContainer#newSwitch(String)}, an
 * ASSwitchStatement allows 'case' and 'default' statements to be added to
 * to the 'switch' statement.
 * 
 * e.g.
 * <pre>
 * ASSwitchStatement switchStmt = method.newSwitch("c");
 * switchStmt.newCase("'a'").addStmt("aay()");
 * switchStmt.newCase("'b'").addStmt("bee()");
 * switchStmt.newDefault().addStmt("cee()");
 * </pre>
 * 
 * Will result in ActionScript something like,
 * <pre>
 * switch(c) {
 * 	case 'a':
 * 		aay();
 * 	case 'b':
 * 		bee();
 * 	default:
 * 		cee();
 * }
 * </pre>
 * 
 * Note that to add 'break' statements, you can use
 * <code>switchStmt.addStmt("break")</code>.
 */
public interface ASSwitchStatement {

	/**
	 * Returns the statements to be executed in the given case for this
	 * switch statement.  Do not call this method more than once with the
	 * same value, as this may result in incorrect code.
	 */
	StatementContainer newCase(String string);

	/**
	 * Returns the statements to be executed in the default case for this
	 * switch statement.  Do not call this method more than once, as this
	 * may result in incorrect code.
	 */
	StatementContainer newDefault();
}
