package uk.co.badgersinfoil.metaas.impl;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.ASClassType;
import uk.co.badgersinfoil.metaas.ASField;
import uk.co.badgersinfoil.metaas.ASMethod;
import uk.co.badgersinfoil.metaas.Visibility;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListToken;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;

public class ASTASClassType extends ASTASType implements ASClassType {

	public ASTASClassType(LinkedListTree clazz) {
		super(clazz);
	}

	public ASMethod newMethod(String name, Visibility visibility, String returnType) {
		ASTASMethod method = ASTFactory.newClassMethod(name, visibility, returnType);
		addMethod(method);
		return method;
	}
	
	public void addMethod(ASTASMethod method) {
		ASTUtils.addChildWithIndentation(findTypeBlock(), method.getAST());
	}

	public String getSuperclass() {
		LinkedListTree ext = ASTUtils.findChildByType(ast, AS3Parser.EXTENDS);
		if (ext == null) return null;
		return ASTUtils.identText(ext.getFirstChild());
	}

	public void setSuperclass(String superclassName) {
		if (superclassName == null) {
			removeExtendsClause();
			return;
		}
		LinkedListTree ext = ASTUtils.findChildByType(ast, AS3Parser.EXTENDS);
		if (ext == null) {
			ext = ASTUtils.newAST(AS3Parser.EXTENDS, "extends");
			ext.appendToken(TokenBuilder.newSpace());
			// hack a space in at the right point,
			LinkedListToken sp = TokenBuilder.newSpace();
			ext.getStartToken().beforeInsert(sp);
			ext.setStartToken(sp);
			ast.addChildWithTokens(2, ext);
		}
		ext.addChildWithTokens(ASTUtils.newIdent(superclassName));
		ext.appendToken(TokenBuilder.newSpace());
	}

	private void removeExtendsClause() {
		for (ASTIterator i=new ASTIterator(ast); i.hasNext(); ) {
			LinkedListTree node = i.next();
			if (node.getType() == AS3Parser.EXTENDS) {
				i.remove();
				break;
			}
		}
	}

	public List getImplementedInterfaces() {
		List results = new LinkedList();
		LinkedListTree impls = ASTUtils.findChildByType(ast, AS3Parser.IMPLEMENTS);
		if (impls != null) {
			for (ASTIterator i=new ASTIterator(impls); i.hasNext(); ) {
				results.add(ASTUtils.identText(i.next()));
			}
		}
		return Collections.unmodifiableList(results);
	}

	public void addImplementedInterface(String interfaceName) {
		LinkedListTree iface = ASTUtils.newIdent(interfaceName);
		LinkedListTree impls = ASTUtils.findChildByType(ast, AS3Parser.IMPLEMENTS);
		if (impls == null) {
			ASTIterator i = new ASTIterator(ast);
			i.find(AS3Parser.TYPE_BLOCK);
			impls = ASTUtils.newAST(AS3Parser.IMPLEMENTS, "implements");
			i.insertBeforeCurrent(impls);
		}
		impls.addChildWithTokens(iface);
	}

	public void removeImplementedInterface(String interfaceName) {
		LinkedListTree impls = ASTUtils.findChildByType(ast, AS3Parser.IMPLEMENTS);
		for (ASTIterator i=new ASTIterator(impls); i.hasNext(); ) {
			LinkedListTree iface = i.next();
			String name = ASTUtils.identText(iface);
			if (name.equals(interfaceName)) {
				i.remove();
				break;
			}
		}
	}

	public ASField newField(String name, Visibility visibility, String type) {
		ASTASField field = ASTFactory.newField(name, visibility, type);
		addField(field);
		return field;
	}
	
	public void addField(ASTASField field) {
		ASTUtils.addChildWithIndentation(findTypeBlock(), field.getAST());
	}

	public ASField getField(String name) {
		for (ASTIterator i=blockIter(); i.hasNext(); ) {
			LinkedListTree member = i.next();
			if (member.getType() == AS3Parser.VAR_DEF) {
				ASField field = new ASTASField(member);
				if (field.getName().equals(name)) {
					return field;
				}
			}
		}
		return null;
	}

	public List getFields() {
		List results = new LinkedList();
		for (ASTIterator i=blockIter(); i.hasNext(); ) {
			LinkedListTree member = i.next();
			if (member.getType() == AS3Parser.VAR_DEF) {
				results.add(new ASTASField(member));
			}
		}
		return Collections.unmodifiableList(results);
	}

	public void removeField(String name) {
		for (ASTIterator i=blockIter(); i.hasNext(); ) {
			LinkedListTree member = i.next();
			if (member.getType() == AS3Parser.VAR_DEF) {
				ASField field = new ASTASField(member);
				if (field.getName().equals(name)) {
					i.remove();
					return;
				}
			}
		}
	}
}
