package uk.co.badgersinfoil.metaas.impl;

import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.ASField;
import uk.co.badgersinfoil.metaas.SyntaxException;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListToken;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;

// TODO: need to handle multi-field declarations like,
//       class X { var foo:String, bar:Number; }

public class ASTASField extends ASTASMember implements ASField {

	private static int INDEX_DEF = 1;
	private static int INDEX_DECL = 2;

	public ASTASField(LinkedListTree ast) {
		super(ast);
	}

	public String getName() {
		return findDecl().getText();
	}

	public void setName(String name) {
		if (name.indexOf('.') != -1) {
			throw new SyntaxException("field name must not contain '.'");
		}
		LinkedListTree newName = ASTUtils.newAST(AS3Parser.IDENT, name);
		ast.setChild(INDEX_DECL, newName);
	}

	public void setInitializer(String expr) {
		LinkedListTree exp = ASTUtils.parseExpr(expr);
		LinkedListTree decl = findDecl();
		LinkedListTree init = ASTUtils.findChildByType(decl, AS3Parser.ASSIGN);
		if (init == null) {
			init = ASTUtils.newAST(AS3Parser.ASSIGN, "=");
			decl.addChildWithTokens(init);
		}
		init.addChildWithTokens(exp);
	}

	public String getDocComment() {
		return DocCommentUtils.getDocComment(ast);
	}

	public void setDocComment(String text) {
		DocCommentUtils.setDocComment(ast, text);
	}

	public boolean isConst() {
		return ast.getChild(INDEX_DEF).getType() == AS3Parser.CONST;
	}

	public void setConst(boolean isConst) {
		if (isConst() == isConst) {
			return;
		}
		LinkedListToken node;
		if (isConst) {
			node = TokenBuilder.newCost();
		} else {
			node = TokenBuilder.newVar();
		}
		ast.setChild(INDEX_DEF, new LinkedListTree(node));
	}

	public String getType() {
		LinkedListTree decl = findDecl();
		LinkedListTree typeSpec = decl.getFirstChild();
		if (typeSpec == null) return null;
		return ASTUtils.typeSpecText(typeSpec);
	}

	private LinkedListTree findDecl() {
		return (LinkedListTree)ast.getChild(INDEX_DECL);
	}

	public void setType(String typeName) {
		LinkedListTree decl = findDecl();
		LinkedListTree typeSpec = decl.getFirstChild();
		if (typeSpec == null) {
			if (typeName != null) {
				decl.addChildWithTokens(ASTUtils.newTypeSpec(typeName));
			}
		} else {
			if (typeName == null) {
				decl.deleteChild(0);
			} else {
				decl.setChildWithTokens(0, ASTUtils.newTypeSpec(typeName));
			}
		}
	}
}