package uk.co.badgersinfoil.metaas.impl;

import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.ASBlock;
import uk.co.badgersinfoil.metaas.ASIfStatement;
import uk.co.badgersinfoil.metaas.ASSwitchStatement;
import uk.co.badgersinfoil.metaas.StatementContainer;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


public class ASTASIfStatement implements ASIfStatement {

	private LinkedListTree ast;

	public ASTASIfStatement(LinkedListTree ast) {
		assertType(ast, AS3Parser.IF);
		this.ast = ast;
	}

	private static void assertType(LinkedListTree ast, int type) {
		if (ast.getType() != type) {
			throw new IllegalArgumentException(ASTUtils.tokenName(ast));
		}
	}

	private StatementContainer getThen() {
		return new ASTStatementList((LinkedListTree)ast.getChild(1));
	}

	public ASBlock getElse() {
		LinkedListTree elseStmt = (LinkedListTree)ast.getChild(2);
		if (elseStmt == null) {
			String indent = ASTUtils.findIndent(ast);
			elseStmt = ASTUtils.newAST(AS3Parser.ELSE, "else");
			ast.appendToken(TokenBuilder.newSpace());
			ast.addChildWithTokens(elseStmt);
			elseStmt.appendToken(TokenBuilder.newSpace());
			LinkedListTree block = ASTFactory.newBlock();
			elseStmt.addChildWithTokens(block);
			ASTUtils.increaseIndentAfterFirstLine(block, indent);
		}
		return new ASTStatementList(elseStmt.getFirstChild());
	}

	public void addStmt(String statement) {
		getThen().addStmt(statement);
	}

	public void addComment(String text) {
		getThen().addComment(text);
	}

	public ASIfStatement newIf(String condition) {
		return getThen().newIf(condition);
	}

	public StatementContainer newFor(String init, String condition,	String update) {
		return getThen().newFor(init, condition, update);
	}

	public StatementContainer newForIn(String init, String list) {
		return getThen().newForIn(init, list);
	}

	public StatementContainer newForEachIn(String init, String list) {
		return getThen().newForEachIn(init, list);
	}

	public StatementContainer newWhile(String condition) {
		return getThen().newWhile(condition);
	}

	public StatementContainer newDoWhile(String condition) {
		return getThen().newDoWhile(condition);
	}

	public ASSwitchStatement newSwitch(String condition) {
		return getThen().newSwitch(condition);
	}

	public boolean containsCode() {
		return getThen().containsCode();
	}

	public void setThen(ASBlock block) {
		LinkedListTree theBlock = ((ASTStatementList)block).getAST();
		ASTIterator i = new ASTIterator(ast);
		i.next();  // move to condition
		i.next();  // move to then-branch
		i.replace(theBlock);
	}
}