package uk.co.badgersinfoil.metaas.impl;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.ASInterfaceType;
import uk.co.badgersinfoil.metaas.ASMethod;
import uk.co.badgersinfoil.metaas.Visibility;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


public class ASTASInterfaceType extends ASTASType implements ASInterfaceType {

	public ASTASInterfaceType(LinkedListTree ast) {
		super(ast);
	}

	public List getSuperInterfaces() {
		List results = new LinkedList();
		LinkedListTree impls = ASTUtils.findChildByType(ast, AS3Parser.EXTENDS);
		if (impls != null) {
			for (ASTIterator i=new ASTIterator(impls); i.hasNext(); ) {
				results.add(ASTUtils.identText(i.next()));
			}
		}
		return Collections.unmodifiableList(results);
	}

	public void addSuperInterface(String interfaceName) {
		LinkedListTree iface = ASTUtils.newIdent(interfaceName);
		LinkedListTree impls = ASTUtils.findChildByType(ast, AS3Parser.EXTENDS);
		if (impls == null) {
			impls = ASTUtils.newAST(AS3Parser.EXTENDS, "extends");
			ast.addChildWithTokens(1, impls);
		}
		impls.addChildWithTokens(iface);
	}

	public void removeSuperInterface(String interfaceName) {
		LinkedListTree impls = ASTUtils.findChildByType(ast, AS3Parser.EXTENDS);
		for (ASTIterator i=new ASTIterator(impls); i.hasNext(); ) {
			LinkedListTree iface = i.next();
			String name = ASTUtils.identText(iface);
			if (name.equals(interfaceName)) {
				i.remove();
				break;
			}
		}
	}

	public ASMethod newMethod(String name, Visibility visibility, String returnType) {
		ASTASMethod meth = ASTFactory.newInterfaceMethod(name, visibility, returnType);
		addMethod(meth);
		return meth;
	}

	public void addMethod(ASTASMethod meth) {
		ASTUtils.addChildWithIndentation(findTypeBlock(), meth.getAST());
	}
}