package uk.co.badgersinfoil.metaas.impl;

import java.util.ArrayList;
import java.util.List;
import org.antlr.runtime.RecognitionException;
import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.ASPackage;
import uk.co.badgersinfoil.metaas.ASType;
import uk.co.badgersinfoil.metaas.SyntaxException;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


public class ASTASPackage implements ASPackage {
	
	private LinkedListTree ast;

	public ASTASPackage(LinkedListTree ast) {
		this.ast = ast;
	}

	public String getName() {
		LinkedListTree first = ast.getFirstChild();
		if (first.getType() == AS3Parser.IDENTIFIER) {
			return ASTUtils.identText(first);
		}
		return null;
	}

	public void setName(String name) {
		ASTIterator i = new ASTIterator(ast);
		// null name given, so remove any existing name,
		if (name == null && i.next().getType() == AS3Parser.IDENTIFIER) {
			i.remove();
			return;
		}
		// otherwise, add or replace the name given,
		LinkedListTree newName = ASTUtils.newIdent(name);
		if (i.next().getType() == AS3Parser.IDENTIFIER) {
			i.replace(newName);
		} else {
			i.insertBeforeCurrent(newName);
		}
	}

	public void addImport(String name) {
		LinkedListTree imp = createImport(name);
		int pos = findNextImportInsertionPoint();
		ASTUtils.addChildWithIndentation(getPkgBlockNode(), pos, imp);
	}

	private int findNextImportInsertionPoint() {
		ASTIterator i = getPkgBlockIter();
		int index = 0;
		while (i.search(AS3Parser.IMPORT) != null) {
			index = i.getCurrentIndex() + 1;
		}
		return index;
	}

	private LinkedListTree createImport(String name) {
		AS3Parser parser = ASTUtils.parse("import "+name+";");
		try {
			LinkedListTree result = ASTUtils.tree(parser.importDefinition());
			parser.endOfFile();  // assert no trailing data
			return result;
		} catch (RecognitionException e) {
			throw new SyntaxException(e);
		}
	}

	public List findImports() {
		ASTIterator i = getPkgBlockIter();
		LinkedListTree imp;
		List result = new ArrayList();
		while ((imp = i.search(AS3Parser.IMPORT)) != null) {
			result.add(importText(imp));
		}
		return result;
	}

	private String importText(LinkedListTree imp) {
		return ASTUtils.identText(imp.getFirstChild());
	}

	public boolean removeImport(String name) {
		ASTIterator i = getPkgBlockIter();
		LinkedListTree imp;
		while ((imp = i.search(AS3Parser.IMPORT)) != null) {
			if (importText(imp).equals(name)) {
				i.remove();
				return true;
			}
		}
		return false;
	}

	public ASType getType() {
		LinkedListTree block = ASTUtils.findChildByType(ast, AS3Parser.BLOCK);
		LinkedListTree type = ASTUtils.findChildByType(block, AS3Parser.CLASS_DEF);
		if (type != null) {
			return new ASTASClassType(type);
		}
		type = ASTUtils.findChildByType(block, AS3Parser.INTERFACE_DEF);
		if (type != null) {
			return new ASTASInterfaceType(type);
		}
		return null;
	}

	private LinkedListTree getPkgBlockNode() {
		return ASTUtils.findChildByType(ast, AS3Parser.BLOCK);
	}

	private ASTIterator getPkgBlockIter() {
		return new ASTIterator(getPkgBlockNode());
	}
}
