package uk.co.badgersinfoil.metaas.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import uk.co.badgersinfoil.metaas.ASProject;
import uk.co.badgersinfoil.metaas.ASSourceFactory;
import uk.co.badgersinfoil.metaas.CompilationUnit;

// TODO: some checks that types are not created that clash with existing types,
// or at least filenames wont collide in the outputDir, would be good.

public class ASTASProject implements ASProject {
	private List classpath;
	private List compilationUnits = new ArrayList();
	private String outputLocation;
	private ASSourceFactory fact;

	public ASTASProject(ASSourceFactory fact) {
		this.fact = fact;
	}

	public void addClasspathEntry(String classpathEntry) {
		classpath.add(classpathEntry);
	}

	public void removeClasspathEntry(String classpathEntry) {
		classpath.remove(classpathEntry);
	}

	public List getClasspathEntries() {
		return Collections.unmodifiableList(classpath);
	}

	public void addCompilationUnit(CompilationUnit cu) {
		compilationUnits.add(cu);
	}

	public void removeCompilationUnit(CompilationUnit cu) {
		compilationUnits.remove(cu);
	}
	public CompilationUnit newClass(String qualifiedClassName) {
		CompilationUnit cu = fact.newClass(qualifiedClassName);
		addCompilationUnit(cu);
		return cu;
	}
	public CompilationUnit newInterface(String qualifiedInterfaceName) {
		CompilationUnit cu = fact.newInterface(qualifiedInterfaceName);
		addCompilationUnit(cu);
		return cu;
	}

	public List getCompilationUnits() {
		return Collections.unmodifiableList(compilationUnits);
	}

	public String getOutputLocation() {
		return outputLocation;
	}

	public void setOutputLocation(String outputLocation) {
		this.outputLocation = outputLocation;
	}

	public void writeAll() throws IOException {
		AutoImporter autoImporter = new AutoImporter();
		autoImporter.performAutoImport(this);
		for (Iterator i=compilationUnits.iterator(); i.hasNext(); ) {
			CompilationUnit cu = (CompilationUnit)i.next();
			write(outputLocation, cu);
		}
	}

	/**
	 * Writes the ActionScript code in the given CompilationUnit to the
	 * given directory, creating any subfolders for package hierarchy as
	 * appropriate, and deriving the filename from the name of the type
	 * defined by the compilation unit.
	 */
	private void write(String destinationDir, CompilationUnit cu) throws IOException {
		String filename = filenameFor(cu);
		File destFile = new File(destinationDir, filename);
		destFile.getParentFile().mkdirs();
		FileOutputStream os = new FileOutputStream(destFile);
		OutputStreamWriter out = new OutputStreamWriter(os);
		fact.newWriter().write(out, cu);
		out.close();
	}

	private static String filenameFor(CompilationUnit unit) {
		String name;
		if (unit.getPackageName().equals("")) {
			name = unit.getType().getName();
		} else {
			name = unit.getPackageName() + "." + unit.getType().getName();
		}
		return name.replace('.', File.separatorChar) + ".as";
	}
}
