package uk.co.badgersinfoil.metaas.impl;

import uk.co.badgersinfoil.metaas.ASClassType;
import uk.co.badgersinfoil.metaas.ASField;
import uk.co.badgersinfoil.metaas.ASInterfaceType;
import uk.co.badgersinfoil.metaas.ASMember;
import uk.co.badgersinfoil.metaas.ASMethod;
import uk.co.badgersinfoil.metaas.ASPackage;
import uk.co.badgersinfoil.metaas.ASProject;
import uk.co.badgersinfoil.metaas.ASType;
import uk.co.badgersinfoil.metaas.CompilationUnit;

// TODO: promote this to the public API at some point

/**
 * To be implemented by classes that want to participate with {@link ASWalker}
 * in an iteration over the contents of an ASProject.
 */
public interface ASVisitor {
	public void visit(ASProject project);
	public void visit(CompilationUnit cu);
	public void visit(ASPackage pkg);
	/**
	 * called just before {@link #visit(ASClassType)} or
	 * {@link #visit(ASInterfaceType)} (as appropriate) with the same value.
	 */
	public void visit(ASType type);
	public void visit(ASClassType clazz);
	public void visit(ASInterfaceType iface);
	/**
	 * called just before {@link #visit(ASMethod)} or
	 * {@link #visit(ASField)} (as appropriate) with the same value.
	 */
	public void visit(ASMember member);
	public void visit(ASMethod method);
	public void visit(ASField field);
	
	/**
	 * An ASVisitor that provides empty implementations of all methods,
	 * usefull for easy subclassing when not all methods need implementing.
	 */
	public class Null implements ASVisitor {
		public void visit(ASClassType clazz)     { /* no-op */ }
		public void visit(ASField field)         { /* no-op */ }
		public void visit(ASInterfaceType iface) { /* no-op */ }
		public void visit(ASMember member)       { /* no-op */ }
		public void visit(ASMethod method)       { /* no-op */ }
		public void visit(ASPackage pkg)         { /* no-op */ }
		public void visit(ASProject project)     { /* no-op */ }
		public void visit(ASType type)           { /* no-op */ }
		public void visit(CompilationUnit cu)    { /* no-op */ }
	}
}
