package uk.co.badgersinfoil.metaas.impl;

import java.util.Iterator;
import uk.co.badgersinfoil.metaas.ASClassType;
import uk.co.badgersinfoil.metaas.ASField;
import uk.co.badgersinfoil.metaas.ASInterfaceType;
import uk.co.badgersinfoil.metaas.ASMember;
import uk.co.badgersinfoil.metaas.ASMethod;
import uk.co.badgersinfoil.metaas.ASPackage;
import uk.co.badgersinfoil.metaas.ASProject;
import uk.co.badgersinfoil.metaas.ASType;
import uk.co.badgersinfoil.metaas.CompilationUnit;


// TODO: promote this to the public API at some point

public class ASWalker {
	private ASVisitor visitor;

	public ASWalker(ASVisitor visitor) {
		this.visitor = visitor;
	}

	public void walk(ASProject project) {
		visitor.visit(project);
		for (Iterator i=project.getCompilationUnits().iterator(); i.hasNext(); ) {
			ASTCompilationUnit cu = (ASTCompilationUnit)i.next();
			walk(cu);
		}
	}

	public void walk(CompilationUnit cu) {
		visitor.visit(cu);
		walk(cu.getPackage());
	}

	public void walk(ASPackage pkg) {
		visitor.visit(pkg);
		walk(pkg.getType());
	}

	public void walk(ASType type) {
		visitor.visit(type);
		if (type instanceof ASClassType) {
			walk((ASClassType)type);
		}
		if (type instanceof ASInterfaceType) {
			walk((ASInterfaceType)type);
		}
	}

	public void walk(ASClassType clazz) {
		visitor.visit(clazz);
		Iterator i;
		for (i=clazz.getMethods().iterator(); i.hasNext(); ) {
			ASMethod method = (ASMethod)i.next();
			walk((ASMember)method);
			walk(method);
		}
		for (i=clazz.getFields().iterator(); i.hasNext(); ) {
			ASField field = (ASField)i.next();
			walk((ASMember)field);
			walk(field);
		}
	}

	public void walk(ASInterfaceType iface) {
		visitor.visit(iface);
		Iterator i;
		for (i=iface .getMethods().iterator(); i.hasNext(); ) {
			ASMethod method = (ASMethod)i.next();
			walk((ASMember)method);
			walk(method);
		}
	}

	public void walk(ASMember member) {
		visitor.visit(member);
	}

	public void walk(ASMethod method) {
		visitor.visit(method);
	}

	public void walk(ASField field) {
		visitor.visit(field);
	}
}