package uk.co.badgersinfoil.metaas.impl;

import org.asdt.core.internal.antlr.AS3Parser;

import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListToken;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


/**
 * Helpers for dealing with documentation-comments.
 */
class DocCommentUtils {
	private DocCommentUtils() { /* hide ctor */ }

	public static String getDocComment(LinkedListTree node) {
		LinkedListToken tok = findDocCommentToken(node);
		if (tok == null) {
			return null;
		}
		return commentToString(tok);
	}

	private static LinkedListToken findDocCommentToken(LinkedListTree node) {
		LinkedListToken tok=node.getStartToken();
		if (tok == null) {
			return null;
		}
		for (tok=tok.getPrev(); tok!=null; tok=tok.getPrev()) {
			switch (tok.getType()) {
			    case AS3Parser.WS:
			    case AS3Parser.NL:
				continue;
			    case AS3Parser.ML_COMMENT:
				if (tok.getText().startsWith("/**")) {
					return tok;
				}
				// fall though,
			    default:
				return null;
			}
		}
		return null;
	}

	// TODO: need to tie comment indentation etc. to the actual current
	//       indentation level.

	private static String commentToString(LinkedListToken tok) {
		return tok.getText().replaceFirst("\\A/\\*+", "")
		                    .replaceFirst("(?:(?<=[\n\r])[ \t]+)?\\*+/\\Z", "")
		                    .replaceAll("([\n\r])\\s*\\*", "$1");
	}

	public static void setDocComment(LinkedListTree node, String text) {
		LinkedListToken tok = findDocCommentToken(node);
		if (text == null) {
			if (tok != null) {
				LinkedListToken next = tok.getNext();
				LinkedListToken prev = tok.getPrev();
				tok.delete();
				if (next.getType() == AS3Parser.NL) {
					next.getNext();
				}
				if (prev.getType() == AS3Parser.WS) {
					prev.getPrev();
				}
			}
			return;
		}
		String indent = ASTUtils.findIndent(node);
		text = "/**" + text.replaceAll("(\n|\r\n|\r)", "$1"+indent+" *");
		if (!text.endsWith("*")) {
			 text += "*";
		}
		text += "/";
		if (tok == null) {
			LinkedListToken comment = TokenBuilder.newMLComment(text);

			LinkedListToken nl = TokenBuilder.newNewline();

			// TODO: try harder to find the right place/line to
			// insert the comment
			node.getStartToken().beforeInsert(comment);
			comment.beforeInsert(nl);
			if (indent.length() > 0) {
				LinkedListToken indentTok = TokenBuilder.newWhiteSpace(indent);
				comment.beforeInsert(indentTok);
			}
		} else {
			tok.setText(text);
		}
	}

	public static void increaseCommentIndent(LinkedListToken tok, String indent) {
		tok.setText(tok.getText().replaceAll("(\n|\r\n|\r)", "$1"+indent));
	}
}